import { defaultTo, is } from 'ramda';

import { Action as ReduxAction, Middleware } from '@reduxjs/toolkit';
import { ThunkAction, ThunkDispatch } from 'redux-thunk';

import { getAttribute } from '@treejs/auth/selectors';
import { Action } from '@treejs/types/redux/actions';
import { isJSON, isNilOrEmpty } from '@treejs/utils';

import { onError, onRequest, onSuccess } from './slice';
import { FetchActions, IFetchConfig } from './types';
import { resolveParams } from './utils';

type ISettings<C> = {
	BASE_ENDPOINT_URL: string;
	authorization?: {
		headerName?: string;
	};
	onError?: (error: any, setting: IFetchConfig<C>) => ThunkAction<void, any, unknown, ReduxAction<string>>;
};

type IStore = {
	dispatch: ThunkDispatch<any, unknown, ReduxAction<string>>;
	getState: () => any;
};

const resolveErrorState = <C>(store: IStore, action: Action<IFetchConfig<any>>, setting: ISettings<C>, error: any) => {
	store.dispatch(onError(action.payload));
	if (is(Function, setting.onError)) {
		store.dispatch(setting.onError(error, action.payload));
	}
	if (is(Function, action.payload.onError)) {
		action.payload.onError(error);
	}
	return error;
};

const buildHeaders =
	<C>(setting: ISettings<C>) =>
	(store: IStore) => {
		const tokenValue = getAttribute([store.getState(), 'accessToken']);

		let headers = {};
		if (!isNilOrEmpty(tokenValue)) {
			headers = {
				...headers,
				[setting?.authorization?.headerName || 'Authorization']: `Bearer ${tokenValue.toString()}`,
			};
		}

		return headers;
	};

const resolveResponse =
	(response: Response) =>
	async <C>(store: IStore, action: Action<IFetchConfig<C>>, setting: ISettings<C>) => {
		const json = await response.text().then((text: string) => {
			return isJSON(text) ? JSON.parse(text) : text;
		});

		if (!response.ok) {
			return Promise.reject(resolveErrorState<C>(store, action, setting, json));
		}

		store.dispatch(onSuccess(action.payload));
		if (is(Function, action.payload.onSuccess)) {
			await action.payload.onSuccess(json);
		}
		return Promise.resolve(json);
	};

export const apiMiddleware =
	<C>(setting: ISettings<C>): Middleware =>
	(store: IStore) =>
	(next) =>
	async (action: Action<IFetchConfig<C>>) => {
		if (action.type !== FetchActions.FETCH) {
			return next(action);
		}

		if (isNilOrEmpty(action.payload.url)) {
			throw new Error(`URL '${action.payload.url}' is not valid, please specify valid URL!`);
		}

		if (is(Function, action.payload.onRequest)) {
			await action.payload.onRequest();
		}

		store.dispatch(onRequest(action.payload));

		const baseUrl = defaultTo(setting.BASE_ENDPOINT_URL)(action.payload.baseUrl);
		const endpoint = `${baseUrl}${resolveParams(action.payload.url, action.payload.queryParams)}`;
		const headers = buildHeaders<C>(setting)(store);

		try {
			const response = await fetch(endpoint, {
				headers: {
					'Content-Type': 'application/json',
					...headers,
					...action.payload.headers,
				},
				method: action.payload.method.toUpperCase(),
				body: JSON.stringify(action.payload.body),
			});

			return await resolveResponse(response)(store, action, setting);
		} catch (error) {
			return Promise.reject(resolveErrorState<C>(store, action, setting, error));
		}
	};
