import React, { useCallback, useEffect, useState } from 'react';
import { range } from 'ramda';

import {
	add,
	endOfMonth,
	endOfWeek,
	getMonth,
	getWeekOfMonth,
	getWeeksInMonth,
	getYear,
	isValid,
	startOfMonth,
	startOfWeek,
	sub,
} from 'date-fns';
import { FaAngleLeft, FaAngleRight, FaCalendarDay } from 'react-icons/fa/';

import { isNilOrEmpty } from '@treejs/utils';

import { Grid, GridCol } from '../../Grid';
import Title from '../../Title';
import { WEEK_DAYS_SHORT } from '../constants';
import { CALENDAR_VIEW, DetailComponentProps } from '../types';
import CalendarView from './CalendarView';

export type CalendarType = {
	/**
	 * Component for view day detail
	 */
	detail?: React.FunctionComponent<DetailComponentProps>;
	hover?: boolean;
	onDayClick?: (date: Date) => void;
	onRangeChange?: (dateFrom: Date, dateEnd: Date) => void;
	/**
	 * Use this for time column on left side
	 * @requires type={CALENDAR_VIEW.WEEK}
	 */
	timeColumn?: React.FunctionComponent;
	/**
	 * @requires timeColumn with defined value
	 */
	timeColumnWidth?: string;
	title?: string;
	type?: CALENDAR_VIEW;
	value?: Date;
};

const todayDate: Date = new Date();

const Calendar: React.FC<CalendarType> = (props) => {
	const {
		value,
		onDayClick,
		onRangeChange,
		type = CALENDAR_VIEW.MONTH,
		title,
		detail,
		timeColumn,
		timeColumnWidth,
		hover,
	} = props;

	const [currentDate, setCurrentDate] = useState<Date>(isValid(new Date(value)) ? new Date(value) : todayDate);

	const getWeekRangeInMonth = useCallback((): number[] => {
		if (type === CALENDAR_VIEW.MONTH) {
			return [0, getWeeksInMonth(currentDate)];
		} else if (type === CALENDAR_VIEW.WEEK) {
			return [getWeekOfMonth(currentDate), getWeekOfMonth(currentDate) + 1];
		}
	}, [currentDate]);

	const handleChangeRange = useCallback(() => {
		if (!isNilOrEmpty(onRangeChange)) {
			if (type === CALENDAR_VIEW.MONTH) {
				onRangeChange(startOfMonth(currentDate), endOfMonth(currentDate));
			} else if (type === CALENDAR_VIEW.WEEK) {
				onRangeChange(startOfWeek(currentDate), endOfWeek(currentDate));
			}
		}
	}, [currentDate]);

	const handleClickPrev = useCallback(() => {
		if (type === CALENDAR_VIEW.MONTH) {
			setCurrentDate(sub(currentDate, { months: 1 }));
		} else if (type === CALENDAR_VIEW.WEEK) {
			setCurrentDate(sub(currentDate, { weeks: 1 }));
		}
		handleChangeRange();
	}, [currentDate, type]);

	const handleClickNext = useCallback(() => {
		if (type === CALENDAR_VIEW.MONTH) {
			setCurrentDate(add(currentDate, { months: 1 }));
		} else if (type === CALENDAR_VIEW.WEEK) {
			setCurrentDate(add(currentDate, { weeks: 1 }));
		}
		handleChangeRange();
	}, [currentDate, type]);

	const handleClickSetToday = useCallback(() => {
		setCurrentDate(todayDate);
		handleChangeRange();
	}, [todayDate]);

	const handleClickDay = useCallback((date: Date) => {
		setCurrentDate(date);
		if (!isNilOrEmpty(onDayClick)) {
			onDayClick(date);
		}
	}, []);

	useEffect(() => {
		handleChangeRange();
	}, []);

	const weekStart = getWeekRangeInMonth()[0];
	const weekEnd = getWeekRangeInMonth()[1];

	return (
		<>
			{!isNilOrEmpty(title) && <Title size="md">{title}</Title>}
			<Grid>
				<GridCol>
					<div className="text-right pb-2.5 select-none">
						<div className="inline-block px-5">
							<b>{('0' + (getMonth(currentDate) + 1)).slice(-2)}</b> {getYear(currentDate)}
						</div>
						<div className="inline-block cursor-pointer pt-1.5 pr-2" onClick={handleClickPrev}>
							<FaAngleLeft />
						</div>
						<div className="inline-block cursor-pointer text-md" onClick={handleClickSetToday}>
							<FaCalendarDay />
						</div>
						<div className="inline-block cursor-pointer pt-1.5 pl-2" onClick={handleClickNext}>
							<FaAngleRight />
						</div>
					</div>
					{type === CALENDAR_VIEW.WEEK && timeColumn ? React.createElement(timeColumn) : null}
					<div style={{ ...(timeColumnWidth ? { marginLeft: timeColumnWidth } : {}) }}>
						<div className="table w-full table-fixed border-collapse">
							<div className="table-row-group">
								<div className="table-row">
									{WEEK_DAYS_SHORT.map((day, i) => (
										<div className="table-cell uppercase pb-1.5 text-center font-bold" key={i}>
											{day}
										</div>
									))}
								</div>
								{range(weekStart, weekEnd).map((i: number) => (
									<React.Fragment key={i}>
										<div className="table-row">
											<CalendarView
												row={i}
												type={type}
												hover={hover}
												detailComponent={detail}
												onChange={handleClickDay}
												value={currentDate}
												todayDate={todayDate}
												currentDate={currentDate}
											/>
										</div>
									</React.Fragment>
								))}
							</div>
						</div>
					</div>
				</GridCol>
			</Grid>
		</>
	);
};

export default Calendar;
