import React, { useCallback, useEffect, useMemo, useState } from 'react';
import { range } from 'ramda';

import { getDate, getDay, getDaysInMonth, getMonth, getYear, setDate, startOfMonth } from 'date-fns';

import { isNilOrEmpty } from '@treejs/utils';

import { CALENDAR_VIEW, DetailComponentProps } from '../types';
import CalendarCell from './CalendarCell';

type IProps = {
	currentDate: Date;
	detailComponent?: React.FunctionComponent<DetailComponentProps>;
	hover?: boolean;
	onChange?: (date: Date) => void;
	row: number;
	todayDate: Date;
	type?: CALENDAR_VIEW;
	value?: Date;
};

const CalendarView: React.FC<IProps> = (props) => {
	const { type, detailComponent, hover, onChange, currentDate, todayDate, row } = props;

	const [value, setValue] = useState(props.value);

	useEffect(() => {
		if (props.value !== value) {
			setValue(props.value);
		}
	}, [props.value]);

	const firstDayOfMonth = useMemo<number>(() => {
		return startOfMonth(currentDate).getDay() - 1;
	}, [currentDate]);

	const daysInMonth = useMemo<number>(() => {
		return getDaysInMonth(currentDate);
	}, [currentDate]);

	const getStartDayOfWeek = useCallback(
		(rowNum = 0): number => {
			if (type === CALENDAR_VIEW.MONTH) {
				return 7 - firstDayOfMonth + (rowNum - 1) * 7;
			} else if (type === CALENDAR_VIEW.WEEK) {
				return getDate(currentDate) - getDay(currentDate);
			}
		},
		[firstDayOfMonth, currentDate, type]
	);

	const handleChangeDate = useCallback(
		(dayNum?: number) =>
			(date?: Date): void => {
				let newDate = date;
				if (!isNilOrEmpty(dayNum)) {
					newDate = setDate(currentDate, dayNum);
				}
				if (!isNilOrEmpty(onChange)) {
					onChange(newDate);
				}
			},
		[currentDate]
	);

	const isEqualToDate = useCallback(
		(dayNum: number, toDate: Date) => {
			if (
				getYear(currentDate) !== getYear(toDate) ||
				getMonth(currentDate) !== getMonth(toDate) ||
				getDate(setDate(currentDate, dayNum)) !== getDate(toDate)
			) {
				return false;
			}
			return true;
		},
		[currentDate]
	);

	let day = 0;
	if (row !== 0) {
		day = getStartDayOfWeek(row);
	}

	return (
		<>
			{range(0, 7).map((i) => {
				if (
					(firstDayOfMonth <= i && row === 0) ||
					(daysInMonth > day && row !== 0) ||
					type === CALENDAR_VIEW.WEEK
				) {
					day++;
					const isToday = isEqualToDate(day, todayDate);
					const isSelected = isEqualToDate(day, value);
					return detailComponent ? (
						React.createElement(detailComponent, {
							key: i,
							value: setDate(currentDate, day),
							isSelected: isSelected,
							isToday: isToday,
							hover: hover,
							onClick: handleChangeDate(),
						})
					) : (
						<CalendarCell
							key={i}
							hasValue
							value={setDate(currentDate, day)}
							hover={hover}
							isToday={isToday}
							isSelected={isSelected}
							onClick={handleChangeDate(day)}
							border={false}
						>
							{day}
						</CalendarCell>
					);
				} else {
					return <div key={i} className="table-cell" />;
				}
			})}
		</>
	);
};

export default CalendarView;
