import React, { useCallback, useMemo } from 'react';
import { filter, includes } from 'ramda';

import cx from 'classnames';
import { FaTimes } from 'react-icons/fa';
import { useDispatch, useSelector } from 'react-redux';

import { isNilOrEmpty } from '@treejs/utils';

import { closeModal } from '../actions';
import { getModals } from '../selectors';
import { ModalId } from '../types';
import { getModalComponent } from '../utils';

type ModalProps = {
	include?: string[];
};

export const Modals: React.FC<ModalProps> = ({ include }) => {
	const modals = useSelector(getModals);

	const dispatch = useDispatch();

	const handleCloseModal = useCallback(
		(modalId: ModalId) => () => {
			dispatch(closeModal(modalId));
		},
		[]
	);

	const filteredModals = useMemo(() => {
		const modalKeys = Object.keys(modals);
		if (isNilOrEmpty(include)) {
			return modalKeys;
		} else {
			return filter((modalKey) => {
				return includes(modalKey, include);
			}, modalKeys);
		}
	}, [include, modals]);

	const unhandledClick = useCallback((e: React.FormEvent<HTMLDivElement>) => e.stopPropagation(), []);

	if (isNilOrEmpty(modals)) {
		return null;
	}

	return (
		<>
			{filteredModals.map((key) => {
				const modal = modals[key];
				return (
					<div key={key} className="fixed top-0 right-0 bottom-0 left-0 z-10 bg-black-dark bg-opacity-70">
						<div
							className="absolute w-full top-10 bg-white max-w-screen-lg left-0 right-0 ml-auto mr-auto"
							style={{ width: modal.width }}
							onClick={unhandledClick}
						>
							<div
								className={cx(
									'absolute right-0 top-0 cursor-pointer text-center text-4xl text-white bg-gray-500',
									'w-14 h-14 pt-2 flex justify-center'
								)}
								onClick={handleCloseModal(key)}
							>
								<FaTimes />
							</div>
							<div className="text-white bg-primary p-3.5 text-xl">{modal.title}</div>
							<div className="bg-white p-4 min-h-4 max-h-v-7 overflow-auto">
								{React.cloneElement(getModalComponent(key), modal?.properties || {})}
							</div>
						</div>
					</div>
				);
			})}
		</>
	);
};

Modals.displayName = 'Modals';

export default Modals;
