import React, { useCallback, useEffect, useMemo, useState } from 'react';

import cx from 'classnames';
import { FaCircle, FaDotCircle } from 'react-icons/fa';

import { IHTMLElement } from '@treejs/types/form';
import { IOptions } from '@treejs/types/options';
import { isNilOrEmpty } from '@treejs/utils';

import { OptionRenderProps } from '../types';

export type IRadioButtonFieldProps<V = IOptions> = {
	/**
	 * Is called for each item in optionsr
	 */
	OptionRender?: React.ReactElement<OptionRenderProps>;
	horizontal?: boolean;
	options: IOptions[];
	value?: string;
} & IHTMLElement<V>;

const RadioButton: React.FC<IRadioButtonFieldProps> = (props) => {
	const { onClick, onBlur, horizontal = false, title, options, OptionRender } = props;

	const [value, setValue] = useState(null);

	useEffect(() => {
		setValue(props.value);
	}, [props.value]);

	const handleClick = useCallback(
		(option: IOptions): any =>
			(e: React.MouseEvent<HTMLInputElement>): void => {
				if (onClick) {
					onClick(option, e);
				}
				setValue(option.code);
			},
		[]
	);

	const handleBlur = useCallback(
		(option: IOptions): any =>
			(e: React.FocusEvent<HTMLInputElement>): void => {
				if (onBlur) {
					onBlur(option, e);
				}
			},
		[]
	);

	const optionElements = useMemo(() => {
		return options.map(({ code, name }): React.ReactElement => {
			return (
				<div
					className={cx({
						'mb-2 block': !horizontal,
						'inline-block w-full': horizontal,
					})}
					key={code}
				>
					<input
						className="hidden"
						id={`${props.name}_${code}`}
						name={name}
						type="radio"
						onClick={handleClick({ code, name })}
						onBlur={handleBlur({ code, name })}
					/>
					<label htmlFor={`${props.name}_${code}`} className="cursor-pointer text-primary">
						{!isNilOrEmpty(OptionRender) ? (
							React.cloneElement(OptionRender, {
								option: { code, name },
								value,
							})
						) : (
							<div className="flex items-center">
								{value === code ? <FaDotCircle /> : <FaCircle />}
								<span className="ml-1 text-black">{name}</span>
							</div>
						)}
					</label>
				</div>
			);
		});
	}, [value, options, props.name]);

	return (
		<div className="radiobutton-container">
			<div className="block text-black pb-2 field-label">{title}</div>
			{horizontal ? <div className="flex">{optionElements}</div> : optionElements}
		</div>
	);
};

export default RadioButton;
