import React, { useCallback, useEffect, useMemo, useState } from 'react';
import { compose, filter, findIndex, includes, prop, propEq, propOr, toLower } from 'ramda';

import cx from 'classnames';
import { FaTrash } from 'react-icons/fa';

import { STATUS } from '@treejs/types/common';
import { IOptions } from '@treejs/types/options';
import { isNilOrEmpty } from '@treejs/utils';

import Button from '../../Button';
import { Grid, GridCol } from '../../Grid';
import { ModalId } from '../../Modal/types';
import TextField from '../../TextField';

type IProps = {
	autoComplete?: boolean;
	closeModal: (id: ModalId) => void;
	name: ModalId;
	onChange: (options: IOptions) => void;
	onRemove: () => void;
	options: IOptions[];
	selectedOption: IOptions;
};

const SelectBoxModal: React.FC<IProps> = (props) => {
	const { name, options, autoComplete, selectedOption, onChange, onRemove, closeModal } = props;

	const [index, setIndex] = useState(0);
	const [searchResults, setSearchResults] = useState(options);

	const setCurrentOption = useCallback(
		(optionIndex: number) => () => {
			setIndex(optionIndex);
		},
		[]
	);

	const setActiveOption = useCallback(
		(option: IOptions) => () => {
			onChange(option);
			closeModal(name);
		},
		[name]
	);

	const handleKeyUp = useCallback(
		(event: KeyboardEvent): void => {
			let newIndex = index;
			switch (event.key) {
				case 'ArrowDown':
					newIndex += 1;
					break;
				case 'ArrowUp':
					newIndex -= 1;
					break;
				case 'Enter':
					setActiveOption(searchResults[index])();
					return;
				default:
					return;
			}
			event.preventDefault();
			if (newIndex < 0) {
				newIndex = 0;
			} else if (newIndex > searchResults.length - 1) {
				newIndex = searchResults.length - 1;
			}
			setIndex(newIndex);
		},
		[index, searchResults]
	);

	useEffect(() => {
		window.addEventListener('keydown', handleKeyUp);
		return () => {
			window.removeEventListener('keydown', handleKeyUp);
		};
	}, [handleKeyUp]);

	useEffect(() => {
		if (!isNilOrEmpty(options)) {
			if (!isNilOrEmpty(selectedOption)) {
				const optionIndex = findIndex(propEq('code', selectedOption.code), options);
				if (optionIndex !== -1) {
					setIndex(optionIndex);
				}
			}
		}
	}, [options, selectedOption]);

	const removeActiveOption = useCallback((): void => {
		onRemove();
		closeModal(name);
	}, []);

	const handleSearch = useCallback(
		(value: string) => {
			setSearchResults(filter<IOptions>(compose(includes(toLower(value)), toLower, prop('name')))(options));
			setIndex(0);
		},
		[options]
	);

	const items = useMemo(() => {
		if (!isNilOrEmpty(searchResults)) {
			return searchResults.map((option, i) => (
				<div
					key={option.code}
					className={cx('cursor-pointer p-2', {
						'bg-focus': index === i,
						'bg-active-light': propOr(false, 'code', selectedOption) === option.code,
					})}
					onClick={setActiveOption({ code: option.code, name: option.name })}
					onMouseEnter={setCurrentOption(i)}
				>
					{option.name}
				</div>
			));
		} else {
			return <div>Nic k zobrazení</div>;
		}
	}, [searchResults, index]);

	return (
		<div>
			{autoComplete && (
				<div className="pb-4">
					<TextField name="search" title="Hledat" onChange={handleSearch} autoFocus />
				</div>
			)}
			{items}
			<Grid cols={4}>
				<GridCol start={4} end={4}>
					<Button label={<FaTrash />} status={STATUS.warning} onClick={removeActiveOption} />
				</GridCol>
			</Grid>
		</div>
	);
};

export default SelectBoxModal;
