import React, { useCallback, useEffect, useState } from 'react';
import { find, propEq, propOr } from 'ramda';

import { useDispatch } from 'react-redux';

import { ModalId } from '@treejs/components/Modal/types';
import { STATUS } from '@treejs/types/common';
import { IHTMLElement } from '@treejs/types/form';
import { isNilOrEmpty } from '@treejs/utils';

import { closeModal, openModal } from '../Modal/actions';
import { addModalComponent } from '../Modal/utils';
import TextField from '../TextField';
import SelectBoxModal from './components/SelectBoxModal';
import { IOption } from './types';

export type ISelectBoxFieldProps<V = IOption> = {
	autoComplete?: boolean;
	disabled?: boolean;
	options: IOption[];
	status?: STATUS;
	value?: string;
} & IHTMLElement<V>;

const SelectBox: React.FC<ISelectBoxFieldProps> = (props) => {
	const { value, options, onBlur, onChange, name, title, autoComplete, disabled, status } = props;

	const dispatch = useDispatch();

	const [selectedOption, setSelectedOption] = useState<IOption>();

	useEffect(() => {
		if (!isNilOrEmpty(value)) {
			if (!isNilOrEmpty(options)) {
				const newValue = find<any>(propEq('code', value), options);
				setSelectedOption(newValue);
			}
		}
	}, [value, options]);

	const handleBlur = useCallback((_: string, e: React.FocusEvent<HTMLInputElement>): void => {
		if (onBlur) {
			onBlur(selectedOption, e);
		}
	}, []);

	const handleOptionClick = useCallback(
		(option: IOption): void => {
			setSelectedOption(option);
			if (onChange) {
				onChange(option, null);
			}
		},
		[onChange, options]
	);

	const handleOptionRemove = useCallback((): void => {
		setSelectedOption(null);
		if (onChange) {
			onChange(null, null);
		}
	}, [onChange, options]);

	const handleCloseModal = useCallback((id: ModalId) => {
		dispatch(closeModal(id));
	}, []);

	const handleOpenModal = useCallback((): void => {
		addModalComponent(
			name,
			<SelectBoxModal
				autoComplete={autoComplete}
				name={name}
				options={options}
				selectedOption={selectedOption}
				closeModal={handleCloseModal}
				onChange={handleOptionClick}
				onRemove={handleOptionRemove}
			/>
		);
		dispatch(openModal({ id: name, title, width: 300 }));
	}, [selectedOption, name, title, autoComplete, options]);

	return (
		<TextField
			disabled={disabled}
			name={name}
			title={title}
			onClick={handleOpenModal}
			onFocus={handleOpenModal}
			onBlur={handleBlur}
			value={propOr('', 'name', selectedOption)}
			status={status}
			readonly
		/>
	);
};

export default SelectBox;
