import React, { useCallback, useMemo, useState } from 'react';

import cx from 'classnames';
import { History } from 'history';
import { FaBars, FaUserCircle } from 'react-icons/fa';

import { isNilOrEmpty } from '@treejs/utils';

import Modals from '../../Modal/components/Modals';
import Toasters from '../../Toaster/components/Toasters';
import { IMenuItem } from '../types';
import SidebarMenuItem from './SidebarMenuItem';
import UserMenu from './UserMenu';

type IProps = {
	children: React.ReactChild;
	components?: {
		footer?: React.ReactElement;
		logo?: React.ReactElement;
		user?: React.ReactElement;
	};
	enabledMenu?: {
		sidebar?: boolean;
		user?: boolean;
	};
	history: History;
	menuItems?: {
		navigation?: IMenuItem[];
		sidebar?: IMenuItem[];
		user?: IMenuItem[];
	};
};

const Skeleton: React.FC<IProps> = (props) => {
	const { children, components = {}, enabledMenu, menuItems, history } = props;
	const {
		user: userComponent = (
			<div className="text-3xl">
				<FaUserCircle />
			</div>
		),
		logo: logoComponent,
		footer: footerComponent,
	} = components;
	const { sidebar: enabledSidebar, user: enabledUser } = enabledMenu;

	const [sidebarOpened, setSidebarOpened] = useState(false);

	const hideSidebar = useCallback(() => {
		if (sidebarOpened) {
			setSidebarOpened(false);
		}
	}, [sidebarOpened]);

	const openSidebar = useCallback(() => {
		if (!sidebarOpened) {
			setSidebarOpened(true);
		}
	}, [sidebarOpened]);

	const openLink = useCallback(
		(link: string, external: boolean) => () => {
			if (external) {
				window.open(link, '_blank');
			} else {
				history.push(link);
			}
			setSidebarOpened(false);
		},
		[history]
	);

	const sidebarItems = useMemo(() => {
		if (!isNilOrEmpty(menuItems.sidebar)) {
			return menuItems.sidebar.map((menuItem, i) => (
				<SidebarMenuItem key={i} isChild={false} onClick={openLink} {...menuItem} />
			));
		} else {
			return null;
		}
	}, [menuItems.sidebar]);

	const navigationItems = useMemo(() => {
		if (!isNilOrEmpty(menuItems.navigation)) {
			return menuItems.navigation.map(({ label, href, external }, i) => (
				<div key={i} className="navigation-item" onClick={openLink(href, external)}>
					{label}
				</div>
			));
		} else {
			return null;
		}
	}, [menuItems.navigation]);

	return (
		<div className="app">
			<div
				className={cx('page-sidebar', {
					'page-sidebar--visible': sidebarOpened,
					'page-sidebar--invisible': !enabledSidebar,
				})}
			>
				<div className="logo">{logoComponent}</div>
				{sidebarItems}
			</div>
			<div
				className={cx('page-main', {
					'page-main--sidebarOverflow': sidebarOpened,
				})}
				onClick={hideSidebar}
			>
				<div className="navigation">
					{!enabledSidebar && <div className="logo">{logoComponent}</div>}
					{!sidebarOpened && enabledSidebar && (
						<div className="navigation-item icon hamburger" onClick={openSidebar}>
							<FaBars />
						</div>
					)}
					<div className="flex flex-wrap">{navigationItems}</div>
					{enabledUser && (
						<div className="user">
							<UserMenu items={menuItems.user} onClick={openLink}>
								{userComponent}
							</UserMenu>
						</div>
					)}
				</div>
				<div className="content">
					{children}
					<div className="mt-4">{footerComponent}</div>
				</div>
			</div>

			<Modals />
			<Toasters />
		</div>
	);
};

Skeleton.displayName = 'Skeleton';

export default Skeleton;
